#-----------------------------------------------------
# Game_System
# システムを管理するオブジェクトクラス。
# システム面での状態を保存。
# $systemに読み込ませて使用する。
#-----------------------------------------------------

class Game_System
  # 定数
  INIT_POS = Marshal.load(open("data/bin/init_pos")) rescue {map: 1, x: 640, y: 320}    # プレイヤーの初期位置
  # 公開インスタンス変数
  attr_reader   :init_pos                 # プレイヤーの初期位置
  attr_reader   :save_pos                 # プレイヤーのセーブ位置
  attr_accessor :menu_disabled            # メニュー禁止
  attr_accessor :bgm_volume               # BGM/BGS音量
  attr_accessor :se_volume                # ME/SE音量
  attr_accessor :moan_volume              # 喘ぎ声音量
  attr_accessor :lose_scene               # 敗北回想フラグ（1:アリ/2:ナシ/3:選択）
  attr_accessor :auto_message             # オートメッセージフラグ
  attr_accessor :message_speed            # 会話メッセージスピード（1:低速/2:中速/3:高速）
  attr_accessor :battle_speed             # 戦闘メッセージスピード（1:低速/2:中速/3:高速）
  attr_accessor :game_level               # 難易度（0;VERYHARD/1:HARD/2:NORMAL/3:EASY/4:VERYEASY）
  attr_accessor :cut_in                   # カットインフラグ
  attr_accessor :animation                # アニメーションフラグ
  attr_accessor :cursor_speed             # カーソルの移動速度
  attr_accessor :walk_dir                 # マップ歩行設定
  attr_accessor :bind                     # キー配列
  attr_accessor :changeable_bgm           # BGMの変更可否
  attr_accessor :time_for_map             # マップの昼/夜
  attr_accessor :enable_logo_cutin        # マップ移動時カットインロゴの表示許可
  attr_accessor :chapter                  # 攻略したチャプター
  attr_accessor :favo_recollect           # お気に入りした敗北回想のIDリスト
  attr_accessor :battle_setplugin         # 戦闘前にプラグインを設定するか（1:アリ/2:ナシ/3:選択）
  attr_accessor :setplugin_hla            # 戦闘前にプラグイン設定で相手のHLAを表示するか
  attr_accessor :narrow                   # 回想装置用絞り込み条件
  attr_accessor :plugin_narrow            # プラグイン絞り込み条件
  attr_accessor :plugin_preset            # プラグインのプリセット
  attr_accessor :ban_map                  # 禁止されているマップ
  attr_accessor :limit_map                # 制限されているマップ
  attr_accessor :remind_mode              # リマインドモード
  attr_accessor :game_clear               # ゲームクリア
  attr_accessor :on_juke                  # ジュークボックス再生中
  attr_accessor :battle_bk                # 戦闘背景
  attr_accessor :rec_onom                 # 敗北回想でオノマトペを表示するか
  # 初期化
  def initialize
    @init_pos = INIT_POS
    @save_pos = INIT_POS
    @menu_disabled = false
    @bgm_volume = 60
    @se_volume = 80
    @moan_volume = 80
    @lose_scene = 3
    @auto_message = false
    @message_speed = 2
    @battle_speed = 2
    @game_level = 0
    @cut_in = true
    @animation = true
    @cursor_speed = 2
    @walk_dir = 1
    @frame_count = 0
    @changeable_bgm = true
    @time_for_map = Array.new(200, :day)
    @enable_logo_cutin = false
    @chapter = []
    @favo_recollect = []
    @battle_setplugin = 2
    @setplugin_hla = false
    @narrow = {
      favorite: false, \
      play: 0, \
      sub1: 0, \
      sub2: 0, \
      age: 0, \
      hair: 0, \
      bust: 0, \
      clothes: 0, \
      socks: 0, \
      accessory: 0, \
      race: 0, \
      hla: 0, \
      mode: 0
    }
    @plugin_narrow = {
      hla: 0, \
      skill: 0, \
      org: 0, \
      sp: 0, \
      tec: 0, \
      mnd: 0, \
      agi: 0, \
      luk: 0, \
      atk: 0, \
      dfn: 0, \
      charm: 0, \
      palsy: 0, \
      rest: 0, \
      stan: 0, \
      a_o: 0
    }
    @plugin_preset = [
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false], \
      [nil, nil, nil, nil, nil, nil, false]
    ]
    @ban_map = []
    @limit_map = []
    @remind_mode = false
    @game_clear = false #ゲームクリア
    @on_juke = {play: 0, battle: 0, rec: 0}
    @battle_bk = 1
    @rec_onom = true
    reset_keybind
    reset_timelap
  end
  # 前編データ引継ぎ時の未定義変数の定義処理
  def set_undefined_var
    @moan_volume = 80 if @moan_volume.nil?
    @chapter = [] if @chapter.nil?
    @favo_recollect = [] if @favo_recollect.nil?
    @battle_setplugin = 2 if @battle_setplugin.nil?
    @setplugin_hla = false if @setplugin_hla.nil?
    if @narrow.nil?
      @narrow = {
        favorite: false, \
        play: 0, \
        sub1: 0, \
        sub2: 0, \
        age: 0, \
        hair: 0, \
        bust: 0, \
        clothes: 0, \
        socks: 0, \
        accessory: 0, \
        race: 0, \
        hla: 0, \
        mode: 0
      }
    end
    if @plugin_narrow.nil?
      @plugin_narrow = {
        hla: 0, \
        skill: 0, \
        org: 0, \
        sp: 0, \
        tec: 0, \
        mnd: 0, \
        agi: 0, \
        luk: 0, \
        atk: 0, \
        dfn: 0, \
        charm: 0, \
        palsy: 0, \
        rest: 0, \
        stan: 0, \
        a_o: 0
      }
    end
    if @plugin_preset.nil?
      @plugin_preset = [
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false], \
        [nil, nil, nil, nil, nil, nil, false]
      ]
    end
    @ban_map = [] if @ban_map.nil?
    @limit_map = [] if @limit_map.nil?
    @remind_mode = false if @remind_mode.nil?
    @game_clear = false if @game_clear.nil?
    @on_juke = {play: 0, battle: 0, rec: 0} if @on_juke.nil?
    @battle_bk = 1 if @battle_bk.nil?
    @rec_onom = true if @rec_onom.nil?
  end
  # キーバインドのリセット
  def reset_keybind
    @bind = Key.default_keybind
  end
  # キーバインドの変更
  # 変更に失敗した場合falseを、成功した場合trueを返す
  def change_keybind(key, code)
    return false if !Key.display_char(code)
    key_haven = nil
    @bind.each{|k, bind|
      if bind == code
        key_haven = k
        break
      end
    }
    @bind[key_haven] = @bind[key] if !key_haven.nil?
    @bind[key] = code
    return true
  end
  # セーブ前の処理
  def on_before_save
    @save_pos = {map: $player.map_id, x: $player.x, y: $player.y, dir: $player.dir}
    @frames_on_save = @frame_count
  end
  # ロード後の処理
  def on_after_load
    @frame_count = @frames_on_save
  end
  # 難易度の配列
  def difficulties
    return ["NORMAL","EASY","VERYEASY", "SPEED", "SKIP"]
  end
  # プレイ時間のリセット
  def reset_timelap
    @frame_count = 0
  end
  # プレイ時間の更新
  def update_timelap
    @frame_count += 1
  end
  # プレイ時間を文字列で取得
  def playtime_s
    time_s = @frame_count / Const::AppFps
    hour = time_s / (60 ** 2)
    min = time_s % (60 ** 2) / 60
    sec = time_s % 60
    sprintf("%02d:%02d:%02d", hour, min, sec)
  end
end
