#-----------------------------------------------------
# Scene_Battle
# バトルのシーンクラス
#-----------------------------------------------------
class Scene_Battle < Scene_Base
  # 初期化
  def initialize(*ext)
    super
    BattleManager.scene = self
    @spriteset = Spriteset_Battle.new(BattleManager.enemy_id)
    BattleManager.spriteset = @spriteset
    BattleManager.spriteset_releasecommand = Spriteset_ReleaseCommand.new if BattleManager.force?
    if $system.on_juke[:battle] != 0 && BattleManager.on_simulation? #端末中でかつ戦闘シミュBGMが設定されている場合
      Audio.play_bgm(:enemy, sprintf('./data/snd/bgm/%s.wav', Data_MusicList.path($system.on_juke[:battle])))
    elsif !Audio.play_enemy_bgm($enemy.id).nil? #特殊戦闘BGMがある場合
      Audio.play_bgm(:enemy, Audio.play_enemy_bgm($enemy.id))
    end
    if ($enemy.id == 1 || $enemy.id == 79) && !$game_switches[9995]
      @log_window.display_tutorial
    else
      BattleManager.off_tutorial
      @log_window.display_encount($enemy)
      @message_window.add_message(Data_EnemyMessage.start($enemy.id), Data_EnemyMessage.image($enemy.id))
    end
    $player.encount_enemies << $enemy.id
    enemies = $player.encount_enemies.uniq
    $player.trophy << 49 if (0..54).all?{|id| enemies.include?(id)}
    $player.trophy << 66 if (Data_Enemy.second_animaloids & enemies) == Data_Enemy.second_animaloids
    $player.trophy << 89 if (55..122).all?{|id| enemies.include?(id)}
  end
  # チュートリアル前処理
  # チュートリアルスタート
  def tutorial_start
    BattleManager.set_tutorial
    @log_window.clear
    @choices_window.deactive
    @choices_window.visible = false
    input_start
  end
  # チュートリアルキャンセル
  def tutorial_cancel
    @log_window.clear
    @choices_window.deactive
    @choices_window.visible = false
    @log_window.display_encount($enemy)
    @message_window.add_message(Data_EnemyMessage.start($enemy.id), Data_EnemyMessage.image($enemy.id))
  end
  # 全てのウィンドウを作成
  def create_all_windows
    super
    create_command_window # メインコマンド
    create_help_window    # 説明
    create_skill_window   # スキル
    create_item_window    # アイテム
    create_log_window     # 戦闘ログ
    create_message_window # メッセージ
    create_choices_window if ($enemy.id == 1 || $enemy.id == 79) && !$game_switches[9995] # チュートリアル用選択肢
  end
  # メインコマンドウィンドウの作成
  def create_command_window
    @windows[:command] = Window_BattleCommand.new(0, 451, 750)
    @windows[:command].add_command("特技", :ability, method(:command_ability))
    @windows[:command].add_command("攻撃", :attack, method(:command_attack)) if BattleManager.force?
    @windows[:command].add_command("性技", :sex, method(:command_sex))
    @windows[:command].add_command("アイテム", :item, method(:command_item))
    @windows[:command].add_command("降参", :surrender, method(:command_surrender))
    @windows[:command].co_index = 1
    @windows[:command].z = 50
    @windows[:command].visible = false
    @windows[:command].deactive
    @windows[:command].refresh
  end
  # ヘルプウィンドウの作成
  def create_help_window
    @windows[:help] = Window_Help.new(408, 524, 666, 105)
    @windows[:help].visible = false
    @windows[:help].z = 100
  end
  # スキルウィンドウの作成
  def create_skill_window
    @windows[:skill] = Window_BattleSkill.new(405,94,694,402)
    @windows[:skill].set_handler(:ok, method(:skill_ok))
    @windows[:skill].set_handler(:cancel, method(:skill_cancel))
    @windows[:skill].help_window = @windows[:help]
    @windows[:skill].z = 100
    @windows[:skill].visible = false
  end
  # アイテムウィンドウの作成
  def create_item_window
    @windows[:item] = Window_BattleItem.new(405,94,694,402)
    @windows[:item].set_handler(:ok, method(:item_ok))
    @windows[:item].set_handler(:cancel, method(:item_cancel))
    @windows[:item].help_window = @windows[:help]
    @windows[:item].z = 100
    @windows[:item].visible = false
  end
  # ログウィンドウの作成
  def create_log_window
    @log_window = Window_BattleLog.new
    @log_window.z = 200
    BattleManager.log_window = @log_window
  end
  # メッセージウィンドウの作成
  def create_message_window
    @message_window = Window_BattleMessage.new
    @message_window.z = 200
    BattleManager.message_window = @message_window
  end
  # チュートリアル用選択肢ウィンドウの作成
  def create_choices_window
    @choices_window = Window_BattleChoices.new
    @choices_window.add_command("はい", :ok, method(:tutorial_start))
    @choices_window.add_command("いいえ", :no, method(:tutorial_cancel))
    @choices_window.z = 200
    @choices_window.visible = true
    @choices_window.active
    @choices_window.log_window = @log_window
  end
  # 入力の開始
  def input_start
    @windows[:command].refresh if BattleManager.force?
    @windows[:command].visible = true
    @windows[:command].active
  end
  # ターンの開始
  def turn_start
    @windows[:help].visible = false
    BattleManager.turn_start
  end
  # 攻撃
  def command_attack
    return if @spriteset.hidden_console?
    if @windows[:command].tutorial_enable(1)
      Audio.play_buzzer
      return
    end
    @windows[:command].deactive
    @spriteset.activate_selecter
    @windows[:help].visible = true
    @windows[:skill].category(:attack)
    @windows[:skill].get_select(:attack)
    @windows[:skill].visible = true
    @windows[:skill].active
    @windows[:skill].refresh
  end
  # 性技ウィンドウ表示
  def command_sex
    return if @spriteset.hidden_console?
    if (BattleManager.force? && BattleManager.position_index == 0)
      Audio.play_buzzer
      return
    end
    if @windows[:command].tutorial_enable(2)
      Audio.play_buzzer
      return
    end
    @windows[:command].deactive
    @spriteset.activate_selecter
    @windows[:help].visible = true
    @windows[:skill].category(:sex)
    @windows[:skill].get_select(:sex)
    @windows[:skill].visible = true
    @windows[:skill].active
    @windows[:skill].refresh
  end
  # 特技ウィンドウ表示
  def command_ability
    return if @spriteset.hidden_console?
    if @windows[:command].tutorial_enable(0)
      Audio.play_buzzer
      return
    end
    @windows[:command].deactive
    @spriteset.activate_selecter
    @windows[:help].visible = true
    @windows[:skill].category(:ability)
    @windows[:skill].get_select(:ability)
    @windows[:skill].visible = true
    @windows[:skill].refresh
    @windows[:skill].active
  end
  # アイテムウィンドウ表示
  def command_item
    return if @spriteset.hidden_console?
    if BattleManager.on_tutorial && BattleManager.turn != 10
      Audio.play_buzzer
    else
      @windows[:command].deactive
      @spriteset.activate_selecter
      @windows[:help].visible = true
      @windows[:item].category(:uses)
      @windows[:item].visible = true
      @windows[:item].refresh
      @windows[:item].active
    end
  end
  # 降参
  def command_surrender
    return if @spriteset.hidden_console?
    if BattleManager.on_tutorial && BattleManager.turn != 10
      Audio.play_buzzer
    else
      BattleManager.battler[:player].make_action_surrender
      @windows[:command].deactive
      turn_start
    end
    #--- 降参処理 ---#
  end
  # スキルを決定
  def skill_ok
    return if @spriteset.hidden_console?
    return if @windows[:skill].item.nil?
    return if !@windows[:skill].enable?(@windows[:skill].item)
    @windows[:skill].deactive
    @spriteset.deactivate_selecter
    @windows[:skill].visible = false
    BattleManager.battler[:player].make_action(@windows[:skill].item)
    turn_start
    #--- スキル使用処理 ---#
  end
  # スキルキャンセル
  def skill_cancel
    return if @spriteset.hidden_console?
    @windows[:skill].select(0)
    @spriteset.deactivate_selecter
    @windows[:help].visible = false
    @windows[:skill].visible = false
    @windows[:skill].deactive
    @windows[:command].active
  end
  # アイテムを使用
  def item_ok
    return if @spriteset.hidden_console?
    if @windows[:item].item.nil?
      Audio.play_buzzer
      return
    end
    @windows[:item].deactive
    @spriteset.deactivate_selecter
    @windows[:item].visible = false
    BattleManager.battler[:player].make_action(@windows[:item].item)
    @windows[:item].uses_item
    turn_start
    #--- アイテム使用処理 ---#
  end
  # アイテムキャンセル
  def item_cancel
    return if @spriteset.hidden_console?
    @windows[:item].select(0)
    @spriteset.deactivate_selecter
    @windows[:help].visible = false
    @windows[:item].visible = false
    @windows[:item].deactive
    @windows[:command].active
  end
  # ウェイト中か？
  def wait?
    super || @log_window.busy? || @message_window.busy? || BattleManager.phase != :input
  end
  # 更新
  def update
    if @spriteset.enemy.initializing?
      @spriteset.update
    else
      BattleManager.update
      @spriteset.display_console if Key.push?(:log)
      super
    end
  end
  # ウィンドウの更新
  def update_all_windows
    return if @spriteset.hidden_console?
    @windows.each{|key, window| window.update }
  end
  # 基本更新
  def update_basic
    return if @spriteset.enemy.initializing?
    super
    unless @spriteset.hidden_console?
      @message_window.update
      @choices_window.update if @choices_window
      @log_window.update if !@message_window.busy? rescue nil
    end
    @spriteset.update
  end
  # 解放処理
  def dispose
    super
    @message_window.dispose
    @log_window.dispose rescue nil
    @choices_window.dispose rescue nil
    @message_window = nil
    @log_window = nil
    @choices_window = nil
    @spriteset.dispose
    if BattleManager.phase != :recollect
      @spriteset.enemy_dispose
      @spriteset.onom_dispose
      @spriteset = nil
    elsif !$system.rec_onom
      @spriteset.onom_dispose
    end
  end
end
