#-----------------------------------------------------
# Scene_CG
# CGの表示を行うシーンクラス
#-----------------------------------------------------
class Scene_CG < Scene_Base
  def initialize(array)
    @spriteset = nil
    # データの整合性確認
    # Spriteset_Enemyが渡された場合(配列の全要素を確認)
    array.each{|data|
      if data.instance_of?(Spriteset_Enemy)
        @spriteset = data
        break
      end
    }
    # Spriteset_Enemyが渡されなかった場合
    if @spriteset.nil?
      @spriteset = Spriteset_Enemy.new(array[0], array[1])
      @spriteset.position = array[1]
      @spriteset.wear = array[2]
      @spriteset.pant = array[3]
      BattleManager.set_mode(Data_Enemy.system(array[0]))
    end
    if $system.on_juke[:rec] != 0
      Audio.play_bgm(:enemy, sprintf('./data/snd/bgm/%s.wav', Data_MusicList.path($system.on_juke[:rec])))
    else
      Audio.play_recollect(@spriteset.enemy_id)
    end
    super
  end
  # 全てのウィンドウを作成
  def create_all_windows
    super
    create_help_window
    create_console_window
    create_motion_window
  end
  # ヘルプウィンドウを作成
  def create_help_window
    @windows[:help] = Window_Help.new(Const::AppWidth - 300, 0, 300, 50)
    @windows[:help].z = 100
    @windows[:help].chg_bgcolor = [150, 0, 0, 0]
  end
  # コンソールウィンドウを作成
  def create_console_window
    @windows[:console] = Window_Help.new(0, 0, 370, 150)
    @windows[:console].z = 100
    @windows[:console].set_text("↑↓：シーン切り替え\n←→：表情切り替え\n会話ログキー：コンソール非表示\nキャンセルキー：戻る")
    @windows[:console].chg_bgcolor = [150, 0, 0, 0]
  end
  # 選択コマンドウィンドウの作成
  def create_motion_window
    @windows[:motion] = Window_CG.new(0, -200)
    if BattleManager.force? && @spriteset.position == 0
      @windows[:motion].add_command("", :stand_0)
      @windows[:motion].add_command("", :stand_1)
      @windows[:motion].add_command("", :stand_2)
      @windows[:motion].add_command("", :stand_3)
      @windows[:motion].add_command("", :attack_0)
      @windows[:motion].add_command("", :attack_1)
      @windows[:motion].add_command("", :attack_2)
      @windows[:motion].add_command("", :attack_3)
    else
      @windows[:motion].add_command("", :stand_0)
      @windows[:motion].add_command("", :stand_1)
      @windows[:motion].add_command("", :stand_2)
      @windows[:motion].add_command("", :stand_3)
      @windows[:motion].add_command("", :attack_0)
      @windows[:motion].add_command("", :attack_1)
      @windows[:motion].add_command("", :attack_2)
      @windows[:motion].add_command("", :attack_3)
      @windows[:motion].add_command("", :ejec_in_0)
      @windows[:motion].add_command("", :ejec_in_1)
      @windows[:motion].add_command("", :ejec_in_2)
      @windows[:motion].add_command("", :ejec_in_3)
      @windows[:motion].add_command("", :ejec_out_0)
      @windows[:motion].add_command("", :ejec_out_1)
      @windows[:motion].add_command("", :ejec_out_2)
      @windows[:motion].add_command("", :ejec_out_3)
    end
    @windows[:motion].set_handler(:console, method(:display_console))
    @windows[:motion].set_handler(:cancel, method(:return_scene))
    @windows[:motion].help_window = @windows[:help]
    @windows[:motion].spriteset = @spriteset
    @windows[:motion].active
  end
  # コンソール表示／非表示（ヘルプウィンドウの表示／非表示）
  def display_console
    return if VEffect.draw_loading?
    @windows[:help].visible = !@windows[:help].visible?
    @windows[:console].visible = !@windows[:console].visible?
  end
  # 回想部屋に戻る
  def return_scene
    return if VEffect.draw_loading?
    SceneManager.return
  end
  # 基本更新
  def update_basic
    super
    @spriteset.update
  end
  # 解放処理
  def dispose
    super
    Audio.delete_bgm(:enemy)
    Audio.play_bgm_list(:map)
    @spriteset.dispose
    @spriteset = nil
  end
end
