#-----------------------------------------------------
# Scene_Jukebox
# ジュークボックスのシーンクラス
#-----------------------------------------------------
class Scene_Jukebox < Scene_Base
  # 初期化
  def initialize(*ext)
    super
    @chg_bgm_flg = false
    @return_flg = false
    create_background
  end
  # 全てのウィンドウを作成
  def create_all_windows
    super
    create_help_window
    create_musiclist_window
  end
  # ヘルプウィンドウの作成
  def create_help_window
    @windows[:help] = Window_Help.new(25, 575, 1233, 135)
    @windows[:help].set_item("決定キーで再生\n\nAキーで戦闘シミュレーションBGMに設定、Sキーで敗北回想BGMに設定")
  end
  # 曲リストの作成
  def create_musiclist_window
    @windows[:music] = Window_MusicList.new(30,127,1210,388)
    @windows[:music].set_handler(:ok, method(:music_ok))
    @windows[:music].set_handler(:cancel, method(:music_cancel))
    @windows[:music].set_handler(:battle, method(:set_battle))
    @windows[:music].set_handler(:rec, method(:set_rec))
    @windows[:music].up_down_img_xy(635,116,635,524)
    @windows[:music].display_up_down = true
    @windows[:music].help_window = @windows[:help]
    @windows[:music].refresh
    @windows[:music].active
  end
  # 背景を作成
  def create_background
    @bg= Sprite_Pis.new("data/img/menu/jukebox.pis", 0, 0)
  end
  # 曲選択
  def music_ok
    Key.freeze
    if $system.on_juke[:play] == 0
      Audio.fadeout_bgm(:map, 60)
    else
      Audio.fadeout_bgm(:jukebox, 60)
    end
    @chg_bgm_flg = true
    if @windows[:music].item == 0
      $system.on_juke = {play: 0, battle: 0, rec: 0}
    else
      $system.on_juke[:play] = @windows[:music].item
    end
    @windows[:music].refresh
  end
  # ジュークボックスキャンセル
  def music_cancel
    Key.freeze
    @windows[:music].deactive
    Audio.fadeout_bgm(:jukebox, 30) if $system.on_juke[:play] != 0
    @chg_bgm_flg = true
    @return_flg = true
    @windows[:music].refresh
  end
  # 戦闘シミュレーションBGMに設定
  def set_battle
    Audio.play_narrow
    $system.on_juke[:battle] = @windows[:music].item
    @windows[:music].refresh
  end
  # 敗北回想BGMに設定
  def set_rec
    Audio.play_narrow
    $system.on_juke[:rec] = @windows[:music].item
    @windows[:music].refresh
  end
  # 更新
  def update
    @bg.update
    @bg.draw
    chg_bgm_proc if @chg_bgm_flg && Audio.not_fade?
    super
  end
  # 選択されフェードアウトが終了後、BGMを変更（0が選択された場合デフォルトに戻す）
  def chg_bgm_proc
    if @return_flg
      Audio.play_bgm(:map, './data/snd/bgm/科学と欲望渦巻く街.wav') if $system.on_juke[:play] != 0
      $system.on_juke[:play] = 0
      SceneManager.return
    else
      if $system.on_juke[:play] != 0
        Audio.play_bgm(:jukebox, sprintf('./data/snd/bgm/%s.wav', Data_MusicList.path($system.on_juke[:play])))
      else
        Audio.play_bgm(:map, './data/snd/bgm/科学と欲望渦巻く街.wav')
      end
    end
    @chg_bgm_flg = false
    Key.unfreeze
  end
  # 解放
  def dispose
    super
    @bg.dispose
    @bg = nil
  end
end
