#-----------------------------------------------------
# Scene_Plugin
# プラグインのシーンクラス
#-----------------------------------------------------

class Scene_Plugin < Scene_Base
  # 初期化
  def initialize(*ext)
    @before_battle = BattleManager.phase == :init
    super
    create_background
    @sprite_preset_high_hr = Sprite_Base.new(239,218)
    @sprite_preset_low_hr = Sprite_Base.new(239,511)
    @sprite_preset_high_hr.image = Image.load($game_directory + '/data/img/menu/preset_hr.png')
    @sprite_preset_low_hr.image = @sprite_preset_high_hr.image.clone
    @item_index_stack = 0
  end
  # 全てのウィンドウを作成
  def create_all_windows
    super
    create_help_window
    create_player_hla
    #BattleManagerがsetupされている場合
    if @before_battle
      #選択コマンドが3;毎回表示の場合
      if $system.battle_setplugin == 3
        create_select_window  # 選択コマンド
        create_check_window   # 確認
      end
      create_enemy_hla if $system.setplugin_hla # 敵のHLAグラフ表示
    end
    create_command_window
    create_one_or_preset_window
    create_status_window
    create_operation_guide
    create_slot_window
    create_preset_plugin_window
    create_preset_window
    create_preset_operation_window
    create_item_window
    create_narrow_window
    create_material_window
    create_comp_window
  end
  # ヘルプウィンドウの作成
  def create_help_window
    @windows[:help] = Window_Help.new(227, 588, 666, 105)
  end
  #HLAレーダーグラフ(自分)
  def create_player_hla
    @player_hla = Plugin_HLA.new(31,33)
    @player_hla.z = 10
  end
  # BattleManagerがsetupされている場合 ##################################
  # 選択コマンドウィンドウの作成
  def create_select_window
    @windows[:select] = Window_HorzCommand.new(Const::AppWidth/2-130, Const::AppHeight/2+50)
    @windows[:select].add_command("　 はい", :yes, method(:command_setup))
    @windows[:select].add_command("　いいえ", :no, method(:goto_battle))
    @windows[:select].set_handler(:cancel, method(:goto_battle))
    @windows[:select].z = 100
    @windows[:select].bgcolor = [200,0,0,0]
    @windows[:select].create_background
    @windows[:select].active
  end
  # 確認ウィンドウの作成
  def create_check_window
    @windows[:check] = Window_Origin.new(0,Const::AppHeight/2-25,Const::AppWidth,50)
    @windows[:check].z = 100
    @windows[:check].bgcolor = [200,0,0,0]
    @windows[:check].create_background
    @windows[:check].draw_text(Const::AppWidth/2-200,0,"プラグインを設定しますか？")
  end
  #HLAレーダーグラフ(敵)
  def create_enemy_hla
    @enemy_hla = HLA.new(31,33, $enemy, C_RED)
    @enemy_hla.z = 10
  end
  #####################################################################
  # コマンドウィンドウの作成
  def create_command_window
    @windows[:plugincommand] = Window_PluginCommand.new(239,83)
    @windows[:plugincommand].help_window = @windows[:help]
    @windows[:plugincommand].add_command("         装備", :equip, method(:command_equip))
    @windows[:plugincommand].add_command("         合成", :comp, method(:command_comp))
    if @before_battle #BattleManagerがsetupされている場合は戦闘開始に
      @windows[:plugincommand].add_command("      戦闘開始", :battle, method(:goto_battle))
      if $system.battle_setplugin == 3
        @windows[:plugincommand].deactive
      else
        @windows[:plugincommand].active
      end
    else
      @windows[:plugincommand].add_command("       　戻る", :menu, method(:return_scene))
      @windows[:plugincommand].set_handler(:cancel, method(:return_scene))
      @windows[:plugincommand].active
    end
  end
  # 個別設定／プリセット操作選択ウィンドウの作成
  def create_one_or_preset_window
    @windows[:oneorpreset] = Window_Command.new(227,176,280)
    @windows[:oneorpreset].add_command("プラグイン個別設定", :one, method(:command_set_one))
    @windows[:oneorpreset].add_command("プリセットの選択・登録", :preset, method(:command_set_preset))
    @windows[:oneorpreset].set_handler(:cancel, method(:command_oneorpreset_no))
    @windows[:oneorpreset].deactive
  end
  # スロットウィンドウの作成
  def create_slot_window
    @windows[:slot] = Window_Slot.new
    @windows[:slot].add_command("", :slot1)
    @windows[:slot].add_command("", :slot2)
    @windows[:slot].add_command("", :slot3)
    @windows[:slot].add_command("", :slot4)
    @windows[:slot].add_command("", :slot5) if $game_switches[9991]
    @windows[:slot].add_command("", :slot6) if $game_switches[9992]
    @windows[:slot].set_handler(:ok, method(:on_slot_ok))
    @windows[:slot].set_handler(:cancel, method(:on_slot_cancel))
    @windows[:slot].set_handler(:switching, method(:command_one_pre_switching))
    @windows[:slot].set_handler(:narrow, method(:plugin_narrow))
    @windows[:slot].help_window = @windows[:help]
    @windows[:slot].deactive
  end
  # ステータスウィンドウの作成
  def create_status_window
    @windows[:status] = Window_PluginStatus.new(930, 189, 270, 370)
    @windows[:status].hla_graph = @player_hla
    @windows[:status].deactive
  end
  # プリセット操作説明ウィンドウ
  def create_operation_guide
    @guide = Sprite_Base.new(410, 520)
    @guide.image = Image_Message.new(460, 32)
    @guide.image.default_size = 32
    @guide.image.draw_text(0, 0, "　")
  end
  # プリセットプラグインリストウィンドウの作成
  def create_preset_plugin_window
    @windows[:preplugin] = Window_PresetPlugin.new(550, 185, 270, 370)
    @windows[:preplugin].status_window = @windows[:status]
    @windows[:preplugin].visible = false
    @windows[:preplugin].deactive
  end
  # プリセットウィンドウの作成
  def create_preset_window
    @windows[:preset] = Window_Preset.new(265,227,147,270)
    @windows[:preset].set_handler(:ok, method(:preset_ok))
    @windows[:preset].set_handler(:cancel, method(:preset_cancel))
    @windows[:preset].set_handler(:switching, method(:command_one_pre_switching))
    @windows[:preset].up_down_img_xy(330,222,330,501)
    @windows[:preset].display_up_down = true
    @windows[:preset].plugin_window = @windows[:preplugin]
    @windows[:preset].help_window = @windows[:help]
    @windows[:preset].refresh
    @windows[:preset].visible = false
    @windows[:preset].deactive
  end
  # プリセット操作ウィンドウの作成
  def create_preset_operation_window
    @windows[:presetoperation] = Window_PresetOperation.new(360,176)
    @windows[:presetoperation].add_command("　選択", :operation, method(:command_preset_operation_yes))
    @windows[:presetoperation].add_command("　登録", :operation, method(:command_preset_operation_yes))
    @windows[:presetoperation].add_command("　削除", :operation, method(:command_preset_operation_yes))
    @windows[:presetoperation].set_handler(:cancel, method(:command_preset_oreration_no))
    @windows[:presetoperation].set_handler(:switching, method(:command_one_pre_switching))
    @windows[:presetoperation].help_window = @windows[:help]
    @windows[:presetoperation].visible = false
    @windows[:presetoperation].deactive
  end
  # アイテムウィンドウの作成
  def create_item_window
    @windows[:item] = Window_PluginItem.new(235,180,640,320)
    @windows[:item].set_handler(:ok, method(:item_ok))
    @windows[:item].set_handler(:cancel, method(:item_cancel))
    @windows[:item].set_handler(:switching, method(:command_one_pre_switching))
    @windows[:item].set_handler(:narrow, method(:plugin_narrow))
    @windows[:item].up_down_img_xy(551,173,551,503)
    @windows[:item].display_up_down = true
    @windows[:item].help_window = @windows[:help]
    @windows[:item].slot_window = @windows[:slot]
    @windows[:item].status_window = @windows[:status]
    @windows[:item].visible = false
    @windows[:item].deactive
  end
  # 絞り込みウィンドウを作成
  def create_narrow_window
    @windows[:narrow] = Window_PluginNarrow.new(235,180,640,320)
    @windows[:narrow].set_handler(:cancel, method(:narrow_cancel))
    @windows[:narrow].help_window = @windows[:help]
    @windows[:narrow].visible = false
    @windows[:narrow].up_down_img_xy(551,173,551,503)
    @windows[:narrow].display_up_down = true
    @windows[:narrow].deactive
    @windows[:narrow].refresh
  end
  # 素材ウィンドウの作成
  def create_material_window
    @windows[:material] = Window_PluginMaterial.new(915, 150, 320, 430)
    @windows[:material].visible = false
    @windows[:material].deactive
  end
  # 合成ウィンドウの作成
  def create_comp_window
    @windows[:comp] = Window_PluginComp.new(250,174,640,370)
    @windows[:comp].set_handler(:ok, method(:comp_ok))
    @windows[:comp].set_handler(:cancel, method(:comp_cancel))
    @windows[:comp].up_down_img_xy(551,173,551,550)
    @windows[:comp].display_up_down = true
    @windows[:comp].help_window = @windows[:help]
    @windows[:comp].material_window = @windows[:material]
    @windows[:comp].visible = false
    @windows[:comp].deactive
  end
  # 背景を作成
  def create_background
    @bg= Sprite_Pis.new("data/img/menu/plugin.pis", 0, 0)
  end
  # 更新
  def update
    @bg.update
    @bg.draw
    if @windows[:slot].active? || @windows[:item].active?
      @sprite_preset_low_hr.update
      @sprite_preset_low_hr.draw
      @guide.update
      @guide.draw
    elsif @windows[:presetoperation].visible?
      @sprite_preset_high_hr.update
      @sprite_preset_low_hr.update
      @sprite_preset_high_hr.draw
      @sprite_preset_low_hr.draw
      @guide.update
      @guide.draw
    end
    super
  end
  # 基本更新
  def update_basic
    super
    @player_hla.update
    @enemy_hla.update if @before_battle && $system.setplugin_hla
  end
  # BattleManagerがsetupされている場合 ##################################
  # プラグインを設定する
  def command_setup
    @windows[:check].visible = false
    @windows[:select].visible = false
    @windows[:select].deactive
    @windows[:plugincommand].active
  end
  #####################################################################
  # 装備(個別設定／プリセット操作選択)
  def command_equip
    @windows[:plugincommand].deactive
    @windows[:oneorpreset].select(0)
    @windows[:oneorpreset].active
    @windows[:help].set_text("プラグインの個別設定をするか、\nプリセットを設定するか選んでください。")
  end
  # 個別設定
  def command_set_one
    @guide.image.clear
    @guide.image.draw_text(0, 0, Vocab::OnesetGuide)
    @windows[:oneorpreset].visible = false
    @windows[:oneorpreset].deactive
    @windows[:item].visible = true
    @windows[:item].refresh
    @windows[:slot].active
  end
  # プリセット設定
  def command_set_preset
    @guide.image.clear
    @guide.image.draw_text(240, 0, Vocab::PresetGuide)
    @windows[:oneorpreset].visible = false
    @windows[:oneorpreset].deactive
    @windows[:preset].visible = true
    @windows[:preplugin].visible = true
    @windows[:presetoperation].select(0)
    @windows[:presetoperation].visible = true
    @windows[:presetoperation].active
  end
  # 個別設定／プリセット操作選択ウィンドウキャンセル
  def command_oneorpreset_no
    @windows[:oneorpreset].deactive
    @windows[:plugincommand].active
  end
  # 個別設定画面⇔プリセット設定画面切り替え
  def command_one_pre_switching
    Audio.play_ok
    @windows[:status].refresh
    if @windows[:presetoperation].visible?
      @windows[:preset].visible = false
      @windows[:preplugin].visible = false
      @windows[:presetoperation].visible = false
      @player_hla.reference = nil
      @windows[:preset].deactive
      @windows[:presetoperation].deactive
      command_set_one
    else
      @windows[:slot].slot_index = nil
      @windows[:item].visible = false
      @player_hla.reference = nil
      @windows[:item].deactive
      @windows[:slot].deactive
      command_set_preset
    end
  end
  # プリセット選択
  def command_preset_operation_yes
    @windows[:presetoperation].deactive
    @windows[:preset].operation = @windows[:presetoperation].index
    @windows[:preset].active
  end
  # プリセット操作ウィンドウキャンセル
  def command_preset_oreration_no
    @windows[:preset].visible = false
    @windows[:preplugin].visible = false
    @windows[:presetoperation].visible = false
    @windows[:presetoperation].deactive
    @windows[:oneorpreset].visible = true
    @windows[:oneorpreset].active
    @windows[:help].set_text("プラグインの個別設定をするか、\nプリセットを設定するか選んでください。")
  end
  # プリセット選択
  def preset_ok
    case @windows[:presetoperation].index
    when 0 # プリセット選択
      $system.plugin_preset[@windows[:preset].index].each_with_index{|plugin_id,idx|
        next if idx == 6
        $player.forget_skill(Data_Plugin.skill($player.equip_plugin[idx]))
        $player.equip_plugin[idx] = plugin_id
      }
      Audio.play_plugin
      self.refresh
      $system.plugin_preset.each_with_index{|plugin, i|
        plugin[6] = @windows[:preset].index == i
      }
      @windows[:slot].refresh
      @windows[:status].refresh
    when 1 # プリセット登録
      6.times{|n|
        $system.plugin_preset[@windows[:preset].index][n] = $player.equip_plugin[n]
      }
      $system.plugin_preset.each_with_index{|plugin, i|
        plugin[6] = @windows[:preset].index == i
      }
      Audio.play_ok
      @windows[:preplugin].set_plugin(@windows[:preset].index)
    when 2 # プリセット削除
      $system.plugin_preset[@windows[:preset].index] = [nil, nil, nil, nil, nil, nil, false]
      Audio.play_ok
      @windows[:preplugin].set_plugin(@windows[:preset].index)
    end
    @windows[:preset].refresh
  end
  # プリセットキャンセル
  def preset_cancel
    @windows[:preset].deactive
    @windows[:status].refresh
    @player_hla.reference = nil
    @windows[:presetoperation].active
  end
  # 合成
  def command_comp
    @windows[:plugincommand].deactive
    @windows[:oneorpreset].visible = false
    @windows[:status].visible = false
    @windows[:material].visible = true
    @windows[:comp].visible = true
    @windows[:comp].select(0)
    @windows[:comp].refresh
    @windows[:comp].active
  end
  # アイテム一覧表示
  def on_slot_ok
    @windows[:slot].slot_index = @windows[:slot].index
    @windows[:item].slot_index = @windows[:slot].index
    @windows[:slot].deactive
    @windows[:item].index = @item_index_stack if @item_index_stack < @windows[:item].item_max
    @windows[:item].active
    @windows[:item].refresh
  end
  # コマンドに戻る
  def on_slot_cancel
    @windows[:slot].slot_index = nil
    @windows[:item].visible = false
    @windows[:slot].deactive
    @windows[:oneorpreset].visible = true
    @windows[:oneorpreset].active
    @windows[:help].set_text("プラグインの個別設定をするか、\nプリセットを設定するか選んでください。")
  end
  # アイテム装備
  def item_ok
    if @windows[:item].index == 0
      begin
        Audio.play_buzzer if @windows[:item].item.nil?
      rescue
        Audio.play_plugin
        $player.forget_skill(Data_Plugin.skill($player.equip_plugin[@windows[:item].slot_index]))
        $player.equip_plugin[@windows[:item].slot_index] = nil
        self.refresh
      end
    else
      if @windows[:item].item
        Audio.play_plugin
        $player.forget_skill(Data_Plugin.skill($player.equip_plugin[@windows[:item].slot_index]))
        $player.equip_plugin[@windows[:item].slot_index] = @windows[:item].item
        self.refresh
      else
        Audio.play_buzzer
      end
    end
    @windows[:slot].refresh
    @windows[:item].refresh
    $system.plugin_preset.each{|plugin| plugin[6] = false}
    @windows[:preset].refresh
    @windows[:status].refresh
    @player_hla.reference = nil
  end
  # アイテムウィンドウキャンセル
  def item_cancel
    @windows[:item].deactive
    @item_index_stack = @windows[:item].index
    @windows[:item].select(0)
    @windows[:item].refresh
    @windows[:status].refresh
    @player_hla.reference = nil
    @windows[:slot].active
  end
  # 絞り込みウィンドウ
  def plugin_narrow
    Audio.play_ok
    if @windows[:slot].active?
      @narrow_return_window = :slot
      @windows[:slot].deactive
    elsif @windows[:item].active?
      @narrow_return_window = :item
    end
    @windows[:item].visible = false
    @windows[:item].deactive
    @windows[:narrow].visible = true
    @windows[:narrow].active
  end
  # 絞り込みウィンドウキャンセル
  def narrow_cancel
    Audio.play_cancel
    @windows[:item].refresh
    @windows[:item].select(0)
    @windows[:narrow].visible = false
    @windows[:narrow].deactive
    @windows[:item].visible = true
    case @narrow_return_window
    when :slot
      @windows[:slot].active
    when :item
      @windows[:item].active
    end
  end
  # アイテム合成
  def comp_ok
    if @windows[:comp].composition?
      @windows[:comp].composition
      @windows[:comp].refresh
    else
      Audio.play_buzzer
    end
  end
  # 合成ウィンドウキャンセル
  def comp_cancel
    @windows[:comp].visible = false
    @windows[:material].visible = false
    @windows[:status].visible = true
    @windows[:oneorpreset].visible = true
    @windows[:comp].deactive
    @windows[:plugincommand].active
  end
  # 終了
  def return_scene
    if SceneManager.stacks.last.instance_of?(Scene_Menu)
      SceneManager.stacks.last.spriteset_status.refresh_meters
      SceneManager.stacks.last.spriteset_status.refresh_points
    end
    SceneManager.return
  end
  # 終了（戦闘へ移行）
  def goto_battle
    VEffect.start_draw_loading
    SceneManager.goto(Scene_Battle)
  end
  # 解放
  def dispose
    super
    @bg.dispose
    @sprite_preset_high_hr.image.dispose
    @sprite_preset_high_hr.dispose
    @sprite_preset_low_hr.image.dispose
    @sprite_preset_low_hr.dispose
    @guide.image.dispose
    @guide.dispose
    @guide.vanish
    @player_hla.dispose
    @bg = nil
    @sprite_preset_high_hr = nil
    @sprite_preset_low_hr = nil
    @guide = nil
    @player_hla = nil
    if @before_battle && $system.setplugin_hla
      @enemy_hla.dispose
      @enemy_hla = nil
    end
    @state = nil
  end
  # プレイヤーのリフレッシュ
  def refresh
    $player.refresh_plugin_buffs
    $player.refresh_plugin_skills
    $player.recovery_hp
    $player.recovery_ep
  end
end
