#-----------------------------------------------------
# Scene_Recollect
# 敗北回想のシーンクラス
#-----------------------------------------------------
class Scene_Recollect < Scene_Base
  ONOM_RESET = 120
  # 初期化
  # ==================================================
  # SceneManagerから初期化された場合、引数は必ずArrayになる
  # ==================================================
  # BattleManagerからcallされた場合は
  # Spriteset_Enemyを格納した配列が渡される
  # ==================================================
  # イベントや回想部屋からcallされた場合は
  # [enemy_id(Integer), position_index(Integer)]
  # の形で渡される
  # ==================================================
  def initialize(array)
    @spriteset = nil
    @spriteset_onom = nil
    @onom_counter = 0
    @on_room = false
    @finish_proc_start = false
    @display_console = true
    # データの整合性確認
    # Spriteset_Enemyが渡された場合(配列の全要素を確認)
    array.each{|data|
      if data.instance_of?(Spriteset_Enemy)
        @spriteset = data
        @spriteset.position = 1 if BattleManager.force? && @spriteset.position == 0
        @spriteset.expression = 0
        @spriteset.attack = 0
      elsif data.instance_of?(Spriteset_Motions)
        @spriteset_onom = data
        break
      end
    }
    # Spriteset_Enemyが渡されなかった場合
    if @spriteset.nil?
      VEffect.start_draw_loading
      @spriteset = Spriteset_Enemy.new(array[0], array[1])
      @spriteset.position = array[1]
      @spriteset.wear = array[2]
      @spriteset.pant = array[3]
      BattleManager.set_mode(Data_Enemy.system(array[0]))
      if $system.animation && $system.rec_onom && BattleManager.get_mode != :event
        @spriteset_onom = Spriteset_Motions.new
        @spriteset_onom.z = 10
      end
      @on_room = true
      $player.encount_enemies << array[0]
      enemies = $player.encount_enemies.uniq
      $player.trophy << 49 if (0..54).all?{|id| enemies.include?(id)}
      $player.trophy << 66 if (Data_Enemy.second_animaloids & enemies) == Data_Enemy.second_animaloids
      $player.trophy << 89 if (55..122).all?{|id| enemies.include?(id)}
    end
    super
    @reco_data = Data_EnemyRecollect.data(@spriteset.enemy_id, @spriteset.position)
    @reco_size = @reco_data.size
    @index = 0
    message_display if $system.lose_scene == 1 || BattleManager.get_mode == :event || SceneManager.stacks.last.instance_of?(Scene_RecollectRoom)
    if $system.on_juke[:rec] != 0 && BattleManager.on_simulation? #端末中でかつ敗北回想BGMが設定されている場合
      Audio.play_bgm(:enemy, sprintf('./data/snd/bgm/%s.wav', Data_MusicList.path($system.on_juke[:rec])))
    else
      Audio.play_recollect(@spriteset.enemy_id)
    end
    @sprite_mode = Sprite_MessageMode.new(Const::AppWidth-$image_auto.width, Const::AppHeight-$image_auto.height)
    $system.auto_message ? @sprite_mode.activate : @sprite_mode.deactive
    @sprite_mode.z = 201
    # 敗北エンド閲覧回数称号 #
    if BattleManager.get_mode == :defeat
      if $game_variables[41].nil?
        $game_variables[41] = 1
      else
        $game_variables[41] += 1
      end
      if $game_variables[41] >= 10
        $player.trophy << 113
      end
    end
    ########################
  end
  # 全てのウィンドウを作成
  def create_all_windows
    super
    create_message_window   # メッセージ
    return if SceneManager.stacks.last.instance_of?(Scene_RecollectRoom)
    if BattleManager.get_mode == :defeat && ($system.lose_scene == 2 || $system.lose_scene == 3)
      create_select_window  # 選択コマンド
      create_check_window(true)   # 確認
    elsif $system.lose_scene == 3 && !@on_room
      create_select_window  # 選択コマンド
      create_check_window(false)   # 確認
    end
  end
  # 選択コマンドウィンドウの作成
  def create_select_window
    @select_window = Window_Command.new(15, 85, 120)
    @select_window.default_font_size = 32
    @select_window.add_command("はい", :yes, method(:command_reco_yes))
    @select_window.add_command("いいえ", :no, method(:command_reco_no))
    @select_window.set_handler(:cancel, method(:command_reco_no))
    @select_window.z = 100
    @select_window.active
  end
  # 確認ウィンドウの作成
  def create_check_window(defeat)
    @check_window = Window_Origin.new(15,15,Const::AppWidth,64)
    @check_window.z = 100
    @check_window.default_font_size = 32
    if defeat
      @check_window.draw_text(0,0,"敗北エンドを見ますか？\n(会話ログキーでメッセージ非表示)")
    else
      @check_window.draw_text(0,0,"敗北回想を見ますか？\n(会話ログキーでメッセージ非表示)")
    end
  end
  # メッセージウィンドウの作成
  def create_message_window
    @message_window = Window_Message.new
    @message_window.z = 200
    @message_window.visible = false
  end
  # 敗北回想を見る
  def command_reco_yes
    @check_window.visible = false
    @select_window.visible = false
    @select_window.deactive
    message_display
  end
  # 敗北回想を見ない
  def command_reco_no
    Audio.stop_moan
    Audio.delete_bgm(:enemy)
    Audio.play_bgm_list(:map) if BattleManager.on_simulation? || @on_room
    ScreenShot.dispose
    Key.unfreeze
    SceneManager.return
  end
  # 終了処理
  def finish_proc
    Key.freeze
    @finish_proc_start = true
    @message_window.visible = false
    Audio.fadeout_bgm(:enemy, 180)
    VEffect.fadeout(:black,180)
    wait(181)
  end
  # メッセージ表示
  def message_display
    @message_window.visible = true
    @message_window.interpreter = self
    load_data
  end
  # 敗北回想読み込み
  def load_data
    if @reco_size > @index
      @message_window.active
      effect(@reco_data[@index][:command]) rescue nil
      sound(@reco_data[@index][:sound]) rescue nil
      @message_window.add_message(@reco_data[@index][:message], image_path, :bottom) rescue nil
      @index += 1
    else
      finish_proc
    end
  end
  # 顔グラフィックのアドレス
  def image_path
    Dir.glob($game_directory + '/data/img/faces/**/*').each{|file|
      return file if file.include?(@reco_data[@index][:face] + '.pis')
    } rescue nil
  end
  # アニメーション・画面効果
  def effect(command_arr)
    command_arr.each{|command|
      case command
      when :normal
        @spriteset.stand                   # 待機
        @onom_counter = ONOM_RESET
      when :attack
        @spriteset.attack = 1              # 攻撃
        @onom_counter = ONOM_RESET
      when :ejaculation_in
        @spriteset.attack = 2              # 射精（中）
        @onom_counter = ONOM_RESET
      when :ejaculation_out
        @spriteset.attack = 3              # 射精（外）
        @onom_counter = ONOM_RESET
      when :expression1; @spriteset.expression = 0      # 表情1
      when :expression2; @spriteset.expression = 1      # 表情2
      when :expression3; @spriteset.expression = 2      # 表情3
      when :expression4; @spriteset.expression = 3      # 表情4
      when :put_wear; @spriteset.wear = 0               # 着衣
      when :break_wear; @spriteset.wear = 1             # 脱衣
      when :put_pants; @spriteset.pant = 0              # パンツアリ
      when :break_pants                                 # パンツナシ
        @spriteset.wear = 1            # 脱衣を同時に行う
        @spriteset.pant = 1            # パンツナシ
      when :fade_out; VEffect.fadeout(:black,30)        # フェードアウト
      when :fade_in; VEffect.fadein(60)                 # フェードイン
      when :white; VEffect.fadeout(:white,30)           # 白塗りつぶし（明転用）
      when :fadeout_bgm; Audio.fadeout_bgm(:enemy, 60)  # bgm停止
      when :flash; VEffect.flash(25, 20)                # フラッシュ
      end
    }
  end
  # SEの再生
  def sound(se)
    return if se[0].nil?
    if se[0] == "stop"
      Audio.stop_moan
      return
    end
    loop = se[0].start_with?("l_")
    se[0].slice!("l_") if loop
    Audio.play_moan(:event,
                  $game_directory + '/data/snd/se/' + se[0] + ".wav",
                  se[1].nil? ? 100 : se[1].to_i,
                  loop
                )
  end
  # オノマトペの表示
  def show_onom
    @spriteset_onom.load_motion_data(@spriteset.enemy_id, @spriteset.position, @spriteset.attack)
  end
  # ウェイト中か？
  def wait?
    super || @spriteset.initializing?
  end
  # ウィンドウの更新
  def update_all_windows
    super
    if @display_console
      @select_window.update if @select_window
      @message_window.update
      @check_window.update if @check_window
    end
    if wait?
      @wait_duration -= 1
    else
      command_reco_no if @finish_proc_start
    end
  end
  # 基本更新
  def update_basic
    super
    update_input
    @spriteset.update
    if $system.animation && $system.rec_onom && BattleManager.get_mode != :event
      if ONOM_RESET < @onom_counter
        show_onom rescue nil
        @onom_counter = 0
      else
        @onom_counter += 1
      end
      @spriteset_onom.process_motion if @spriteset.sync
      @spriteset_onom.update
    end
    @sprite_mode.update unless @finish_proc_start
  end
  # オプション入力処理
  def update_input
    if Key.push?(:log)
      @display_console ^= true
      if @display_console
        $system.auto_message = @setconsole_prev_auto unless @setconsole_prev_auto.nil?
      else
        @setconsole_prev_auto = $system.auto_message
        $system.auto_message = false
      end
      $system.auto_message ? @sprite_mode.activate : @sprite_mode.deactive
    end
    if Key.push?(:auto)
      $system.auto_message ^= true
      $system.auto_message ? @sprite_mode.activate : @sprite_mode.deactive
    end
  end
  # 解放処理
  def dispose
    super
    @select_window.dispose if @select_window
    @check_window.dispose if @check_window
    @message_window.dispose
    @spriteset.dispose
    @spriteset_onom.dispose if $system.animation && $system.rec_onom && BattleManager.get_mode != :event
    @spriteset = nil
    @spriteset_onom = nil
    @reco_data = nil
    @index = nil
    @finish_proc_start = false
  end
end
