#-----------------------------------------------------
# Spriteset_Battle
# 戦闘のスプライト群を管理するオブジェクト。
#-----------------------------------------------------
class Spriteset_Battle
  # オブジェクト初期化
  def initialize(id)
    @id = id
    @shake = [0, 0, 0, false] # [0]:duration [1]:X強さ [2]:Y強さ [3]:減衰
    @hidden_console = false
    create_sprites
  end
  # スプライトの作成
  def create_sprites
    create_enemy_sprite
    create_battle_back
    create_battle_console
    if $system.animation
      @sprites_balloon = [] if $system.cut_in
      @spriteset_onom = Spriteset_Motions.new
      @spriteset_onom.z = 10
    end
  end
  # オブジェクトの解放
  def dispose
    @cs_back.image.dispose
    @cs_back.dispose
    @vista.image.dispose unless @vista.image.nil?
    @vista.dispose
    ScreenShot.dispose
    @cs_selecter.image.dispose
    @cs_selecter.dispose
    @cs_hla.dispose
    @cs_hp.dispose
    @cs_sp.dispose
    @cs_ep.dispose
    @cs_enemy_hp.dispose
    @cs_state.dispose
    @cs_enemy_state.dispose
    @sprites_balloon.each{|sprite| sprite.dispose} if $system.animation && $system.cut_in
    @cs_back.vanish
    @vista.vanish
    @cs_selecter.vanish
    @sprites_balloon.each{|sprite| sprite.vanish} if $system.animation && $system.cut_in
    @cs_back = nil
    @vista = nil
    @cs_selecter = nil
    @cs_hla = nil
    @cs_hp = nil
    @cs_sp = nil
    @cs_ep = nil
    @cs_enemy_hp = nil
    @cs_state = nil
    @cs_enemy_state = nil
    @sprites_balloon = nil
    @id = nil
    @shake = nil
  end
  # 敵スプライトの解放
  def enemy_dispose
    @spriteset_enemy.dispose
    @spriteset_enemy = nil
  end
  # オノマトペの解放
  def onom_dispose
    @spriteset_onom.dispose if $system.animation
    @spriteset_onom = nil
  end
  # 敵スプライトの作成
  def create_enemy_sprite
    @spriteset_enemy = Spriteset_Enemy.new(@id)
  end
  # 背景の作成
  def create_battle_back
    @vista = Sprite_Base.new(0, 0)
    @vista.center_x = 0
    @vista.center_y = 0
    @vista.z = -1
    # 背景画像設定
    if BattleManager.on_simulation?
      case $system.battle_bk
      when 0 ; @vista.image = nil
      else
        pis = Pis.load($game_directory + '/data/img/battle/' + $system.battle_bk.to_s + '.pis')
        @vista.image = pis[0].change_hls(0, -20, -20)
        @vista.scale_x = 2
        @vista.scale_y = 2
      end
    else
      @vista.image = ScreenShot.get.change_hls(0, -50, -100)
    end
  end
  # バトルコンソールの作成
  def create_battle_console
    @cs_back = Sprite_Base.new(0, 0)
    @cs_back.image = Image.load("data/img/battle/battle.png")
    @cs_back.z = 90
    @cs_hla = HLA.new(1150,592)
    @cs_hla.z = 100
    @cs_hp = Gauge_Hp.new(176, 665)
    @cs_hp.z = 100
    @cs_sp = Gauge_VertEp.new(10, 256)
    @cs_sp.z = 100
    @cs_ep = Sprite_PointEp.new(80, 640)
    @cs_ep.z = 100
    @cs_selecter = Sprite_Base.new(0, 0)
    @cs_selecter.image = Image.load("data/img/battle/select_window.png")
    @cs_selecter.visible = false
    @cs_selecter.z = 90
    @cs_enemy_hp = Gauge_EnemyHp.new(1224, 12)
    @cs_enemy_hp.z = 100
    @cs_state = Sprite_BattleState.new(876, 661)
    @cs_state.subject = $player
    @cs_state.z = 100
    @cs_enemy_state = Sprite_BattleState.new(973, 6)
    @cs_enemy_state.subject = $enemy
    @cs_enemy_state.z = 100
  end
  # メインコマンドのセレクタを表示
  def activate_selecter
    @cs_selecter.visible = true
  end
  # メインコマンドのセレクタを非表示
  def deactivate_selecter
    @cs_selecter.visible = false
  end
  # 敵スプライトセットへのアクセス
  def enemy
    return @spriteset_enemy
  end
  # オノマトペへのアクセス
  def onom
    return @spriteset_onom
  end
  # 更新
  def update
    @vista.update
    @vista.draw
    @spriteset_enemy.update
    if $system.animation
      update_balloons if $system.cut_in
      @spriteset_onom.process_motion if @spriteset_enemy.sync
      @spriteset_onom.update
    end
    unless @hidden_console
      @cs_back.update
      @cs_back.draw
      @cs_selecter.update
      @cs_selecter.draw
      @cs_hla.update
      @cs_hp.update
      @cs_sp.update
      @cs_ep.update
      @cs_ep.draw
      @cs_enemy_hp.update
      @cs_state.update
      @cs_state.draw
      @cs_enemy_state.update
      @cs_enemy_state.draw
    end
    update_shake
  end
  # 更新(敗北回想)
  def update_reco
    @vista.update
    @vista.draw
    @spriteset_enemy.update
  end
  # フキダシの更新
  def update_balloons
    sp = []
    @sprites_balloon.each{|sprite|
      if sprite.active?
        sprite.next_index if @spriteset_enemy.sync
        sprite.update
      else
        sprite.dispose
        sprite.vanish
        sp << sprite
      end
    }
    sp.each{|disposed_sprite|
      @sprites_balloon.delete(disposed_sprite)
    }
  end
  # シェイクの更新
  def update_shake
    if @shake[0] == 0
      @spriteset_enemy.ox = 0
      @spriteset_enemy.oy = 0
      @shake = [0, 0, 0, false]
      return
    end
    @spriteset_enemy.ox = rand(-@shake[1]/2 .. @shake[1]/2)
    @spriteset_enemy.oy = rand(-@shake[2]/2 .. @shake[2]/2)
    if decay_shake?
      @shake[1] -= @shake[1] / @shake[0]
      @shake[2] -= @shake[2] / @shake[0]
    end
    @shake[0] -= 1
  end
  # フキダシの表示
  def show_balloon(skill_id)
    exist = Sprite_BattleBalloon.exist?(skill_id)
    return if !exist
    @sprites_balloon << Sprite_BattleBalloon.new(skill_id, exist)
  end
  # フキダシの停止
  def clear_balloon
    @sprites_balloon.each{|balloon| balloon.deactive} unless @sprites_balloon.nil?
  end
  # 画面シェイクの開始
  def shake(x, y, duration, decay)
    @shake[0] = duration
    @shake[1] = x
    @shake[2] = y
    @shake[3] = decay == 0 ? false : true
  end
  # 画面のシェイク中か？
  def shaking?
    return @shake[0] != 0
  end
  # 画面シェイクの減衰が有効か
  def decay_shake?
    return @shake[3]
  end
  # オノマトペの表示
  def show_onom
    @spriteset_onom.load_motion_data(@id, @spriteset_enemy.position, @spriteset_enemy.attack)
  end
  # コンソールの非表示
  def display_console
    @hidden_console ^= true
    @hidden_console ? Audio.play_ok : Audio.play_buzzer
  end
  # コンソールが非表示か？
  def hidden_console?
    return @hidden_console
  end
end
