#-----------------------------------------------------
# Window_EnemyList
# 敵の名前のリストを表示するリスト
#-----------------------------------------------------
class Window_EnemyList < Window_Selectable
  def initialize(x, y, width, height)
    super(x, y, width, height)
    @enemylist = []
    @encountlist = []
  end
  # 行の高さ
  def line_height
    return 33
  end
  # 桁数の取得
  def col_max
    return 1
  end
  # 属性配列からの絞り込みを行わない例外キー
  def narrow_exception_key
    return [:favorite, :hla, :mode]
  end
  # サブプレイのkey名変換用配列
  def sub_play_key
    return [:sub1, :sub2]
  end
  # リストの含めるか？
  def include?(index)
    # 戦闘シミュレーションで戦闘無しの相手だった場合
    if @scene == :battle
      return false if Data_Enemy.not_enemy?(index) || Data_Enemy.cg_only?(index)
    end
    if @cg_or_event == :event
      return false if Data_Enemy.cg_only?(index)
    end
    #お気に入り絞込み
    if $system.narrow[:favorite]
      return false if !$system.favo_recollect.include?(index)
    end
### ---属性配列からの絞り込み（属性のすべてが不一致時にselectにfalseを返す挙動）--- ###
    property = Data_Enemy.property(index)
    $system.narrow.each{|key, val|
      next if val == 0 || narrow_exception_key.include?(key)
      return false if property.all?{|pro|
                        #絞り込み設定のsub1,sub2はvocabでsubに統一されてるため
                        #絞り込み時に該当しないのでメタ処理
                        if sub_play_key.include?(key)
                          pro != Vocab::EnemyProperty[:sub][val]
                        else
                          pro != Vocab::EnemyProperty[key][val]
                        end
                      }
    }
###---------------------------------------------------------------------------###
### ---他データを参照しての絞り込み--- ###
    #HLA絞り込み
    if $system.narrow[:hla] != 0
      case hla_convert(Data_Enemy.hla(index))
      when "H" ; return false if $system.narrow[:hla] != 1
      when "L" ; return false if $system.narrow[:hla] != 2
      when "A" ; return false if $system.narrow[:hla] != 3
      when "NORMAL" ; return false if $system.narrow[:hla] != 4
      end
    end
    #戦闘形式絞り込み
    if $system.narrow[:mode] != 0
      case Data_Enemy.system(index)
      when :collect ; return false if $system.narrow[:mode] != 1
      when :force ; return false if $system.narrow[:mode] != 2
      when :event ; return false if $system.narrow[:mode] != 3
      when :defeat ; return false if $system.narrow[:mode] != 4
      end
    end
###---------------------------------###
    #すべての絞り込みに該当しなかった場合trueを返す
    return true
  end
  # 選択されている項目が有効か？
  def enabled?(index)
    @encountlist.each{|id| return true if id == index}
    return false
  end
  # 項目数の取得
  def item_max
    return [@enemylist.size, 1].max
  end
  # 敵リストの作成
  def make_enemy_list
    @enemylist = []
    @encountlist = []
    @enemylist = Data_Enemy.all.select{|enemy| include?(enemy[:id])}
    @encountlist = $player.encount_enemies
    @encountlist.uniq!
  end
  # indexにより項目を描画
  def draw_item(index, rect_x, rect_y)
    if @enemylist[index].nil?
      draw_text(rect_x, rect_y, "\\C[9]none")
      return
    end
    if enabled?(@enemylist[index][:id])
      if $system.favo_recollect.include?(@enemylist[index][:id])
        draw_text(rect_x, rect_y, "\\C[5]" + @enemylist[index][:name])
      else
        draw_text(rect_x, rect_y, @enemylist[index][:name])
      end
    else
      draw_text(rect_x, rect_y, "？？？？？")
    end
  end
# 決定処理
  def process_ok
    if @enemylist[index].nil?
      Audio.play_buzzer
    else
      super if enabled?(@enemylist[index][:id])
    end
  end
  # アイテムの取得
  def item
    return nil if @enemylist[index].nil?
    return enabled?(@enemylist[index][:id]) ? @enemylist[index][:id] : nil
  end
  # ヘルプテキスト更新
  def update_help
    case @scene
    when :battle
      if @enemylist.empty? || @enemylist[index].nil? || !enabled?(@enemylist[index][:id])
        @help_window.set_item("戦闘シミュレーションが行えます")
      else
        begin
          help = sprintf("HLAタイプ：%s\n%s", hla_convert(Data_Enemy.hla(@enemylist[index][:id])), Data_Enemy.capture(@enemylist[index][:id]))
        rescue
          select(0)
          help = sprintf("HLAタイプ：%s\n%s", hla_convert(Data_Enemy.hla(@enemylist[index][:id])), Data_Enemy.capture(@enemylist[index][:id]))
        end
        @help_window.set_item(help.nil? ? ' ' : help)
      end
      @sprite.image = help.nil? ? nil : @enemy_image[@enemylist[index][:id]] rescue nil
    when :recollect
      begin
        @sprite.image = enabled?(@enemylist[index][:id]) ? @enemy_image[@enemylist[index][:id]] : nil
      rescue
        @sprite.image = nil
      end
      help = $system.narrow[:favorite] ? "\\C[5]お気に入りのみ\\C[1]\n" : "\n"
      @help_window.set_text(
        help = help + \
        "プレイ：" + Vocab::EnemyProperty[:play][$system.narrow[:play]] +  \
        "　見た目：" + Vocab::EnemyProperty[:age][$system.narrow[:age]] +  \
        "　髪型：" + Vocab::EnemyProperty[:hair][$system.narrow[:hair]] +  \
        "　胸のサイズ：" + Vocab::EnemyProperty[:bust][$system.narrow[:bust]] +  \
        "　服装：" + Vocab::EnemyProperty[:clothes][$system.narrow[:clothes]] +  \
        "\n靴下：" + Vocab::EnemyProperty[:socks][$system.narrow[:socks]] +  \
        "　装飾品：" + Vocab::EnemyProperty[:accessory][$system.narrow[:accessory]] +  \
        "　種族：" + Vocab::EnemyProperty[:race][$system.narrow[:race]] +  \
        "　HLA：" + Vocab::EnemyProperty[:hla][$system.narrow[:hla]] +  \
        "　戦闘形式：" + Vocab::EnemyProperty[:mode][$system.narrow[:mode]]
      )
    end
  end
  # リフレッシュ
  def refresh
    make_enemy_list
    super
    update_help
    select(0) if @enemylist[index].nil?
  end
  # シーン取得
  def scene=(scene)
    @scene = scene
  end
  # CG or 敗北改装
  def cg_or_event=(sym)
    @cg_or_event = sym
  end
  # イメージの取得
  def enemy_image(sprite, image)
    @sprite = sprite
    @enemy_image = image
  end
  # HLAを取得し特化HLAをStringで返す
  def hla_convert(hla)
    unless hla[:h] == hla[:l] && hla[:l] == hla[:a]
      max_hla = hla.max { |a, b| a[1] <=> b[1] }
      case max_hla[0]
      when :h ; return "H"
      when :l ; return "L"
      when :a ; return "A"
      end
    else
      return "NORMAL"
    end
  end
  # カーソルを下に移動
  def cursor_down
    if row < row_max - 1
      select(@index + col_max)
    else
      select(0)
    end
  end
  # カーソルを上に移動
  def cursor_up
    if row + 1 > 1
      select(@index - col_max)
    else
      select(item_max)
    end
  end
  # カーソルを右に移動
  def cursor_right
    select(@index + page_row_max * col_max)
  end
  # カーソルを左に移動
  def cursor_left
    select(@index - page_row_max * col_max)
  end
  # 決定/キャンセル処理、お気に入りを追加
  def process_handling
    super
    return call_handler(:favorite) if Key.push?(K_A)
    return call_handler(:narrow) if Key.push?(K_S)
    return call_handler(:battlebk) if Key.push?(K_D)
    return call_handler(:bgm) if Key.push?(K_W)
  end
  # 解放
  def dispose
    super
    @sprite.image.dispose rescue nil
    @sprite.dispose
    @sprite.vanish
    @enemy_image.each{|enemy| enemy.dispose}
    @sprite = nil
    @enemy_image = nil
    @scene = nil
    @enemylist = nil
    @encountlist = nil
  end
end
