#-----------------------------------------------------
# class Window_Slot
# プラグイン画面のスロットを作成するウィンドウクラス。
#-----------------------------------------------------

class Window_Slot < Window_Command
  attr_accessor :slot_index
  # 初期化
  def initialize(x = 0, y = 168, width = 212)
    super(x, y, width)
    @slot_index = nil
    create_spriteset
    @flasher = {decide: false, dir: 5}
  end
  # 素材のフォルダパス
  def material_path
    return $game_directory + '/data/img/menu/'
  end
  # スプライトセットの生成
  def create_spriteset
    @image_button = Image.load(material_path + 'slot.png')
    @spriteset = {}
    @spriteset[:slot1]    = Spriteset_Slot.new(0, 0)
    @spriteset[:slot2]    = Spriteset_Slot.new(0, 0)
    @spriteset[:slot3]    = Spriteset_Slot.new(0, 0)
    @spriteset[:slot4]    = Spriteset_Slot.new(0, 0)
    @spriteset[:slot5]    = Spriteset_Slot.new(0, 0) if $game_switches[9991]
    @spriteset[:slot6]    = Spriteset_Slot.new(0, 0) if $game_switches[9992]
    set_image_plugin
    @sprite_selecter = {}
    @sprite_selecter[:slot1] = Sprite_Base.new(0, 0)
    @sprite_selecter[:slot2] = Sprite_Base.new(0, 0)
    @sprite_selecter[:slot3] = Sprite_Base.new(0, 0)
    @sprite_selecter[:slot4] = Sprite_Base.new(0, 0)
    @sprite_selecter[:slot5] = Sprite_Base.new(0, 0) if $game_switches[9991]
    @sprite_selecter[:slot6] = Sprite_Base.new(0, 0) if $game_switches[9992]
    set_image_selecter
  end
  # 表示するプラグイン名を設定
  def set_image_plugin
    idx = 0
    @spriteset.each{|k, slot|
      slot.image_button = @image_button
      slot.image_equip_name = $player.equip_plugin[idx].nil? ? "" : Data_Plugin.name($player.equip_plugin[idx])
      slot.padding_y = 23
      idx += 1
    }
  end
  # 表示するタイプ色を設定
  def set_image_selecter
    idx = 0
    @sprite_selecter.each{|k, selecter|
      selecter.image = sym_path_convert(idx)
      selecter.z = @spriteset[:slot1].z + 2
      idx += 1
    }
  end
  # シンボルから選択画像のパスを設定したImageオブジェクトを返す
  def sym_path_convert(idx)
    if $player.equip_plugin[idx].nil?
      image = "select_nil"
    else
      case Data_Plugin.type($player.equip_plugin[idx])
      when :h ;      image = Data_Plugin.gravity($player.equip_plugin[idx]) < 0 ? "select_h_d" : "selecter"
      when :l ;      image = Data_Plugin.gravity($player.equip_plugin[idx]) < 0 ? "select_l_d" : "select_l"
      when :a ;      image = Data_Plugin.gravity($player.equip_plugin[idx]) < 0 ? "select_a_d" : "select_a"
      when :unique ; image = "select_unique"
      when :normal ; image = "select_normal"
      end
    end
    return Image.load(material_path + image + '.png')
  end
  # １項目あたりの高さを取得
  def item_height
    return 88
  end
  # アクティブなコマンドの横偏差
  def padding_active
    return 20
  end
  # 更新
  def update
    item_max.times{|index|
      rect = item_rect_for_text(index)
      key = @list[index][:key]
      if active?
        if !key.nil? && !@spriteset[key].nil?
          @spriteset[key].x = self.x + rect.x + (@index == index ? padding_active : 0)
          @spriteset[key].y = self.y + rect.y
          @sprite_selecter[key].x = @spriteset[key].x + 5
          @sprite_selecter[key].y = @spriteset[key].y + 5
        end
        if index == @index
          @sprite_selecter[key].x = self.x + rect.x + padding_active + 5
          @sprite_selecter[key].y = self.y + rect.y + 5
          if @flasher[:decide]
            @sprite_selecter[key].alpha = 255
          else
            @sprite_selecter[key].alpha = [[@flasher[:dir] + @sprite_selecter[key].alpha, 0].max, 255].min
            if @sprite_selecter[key].alpha == 0 || @sprite_selecter[key].alpha == 255
              @flasher[:dir] *= -1
            end
          end
        else
          @sprite_selecter[key].alpha = 255
        end
      else
        @spriteset[key].x = self.x + rect.x + (@slot_index == index ? padding_active : 0)
        @spriteset[key].y = self.y + rect.y
        @sprite_selecter[key].x = @spriteset[key].x + 5
        @sprite_selecter[key].y = @spriteset[key].y + 5
        if index == @slot_index
          @sprite_selecter[key].alpha = [[@flasher[:dir] + @sprite_selecter[key].alpha, 0].max, 255].min
          if @sprite_selecter[key].alpha == 0 || @sprite_selecter[key].alpha == 255
            @flasher[:dir] *= -1
          end
        else
          @sprite_selecter[key].alpha = 255
        end
      end
    }
    @spriteset.each{|k, spriteset| spriteset.update}
    @sprite_selecter.each{|k, sprite_selecter|
      sprite_selecter.update
      sprite_selecter.draw
    }
    super
  end
  # セレクタのフラッシュ状態の変更
  def flash=(bool)
    @flasher[:decide] = bool
  end
  # ウィンドウの状態更新
  def update_stat
    super
    @cursor.visible = false
  end
  # カーソルを下に移動
  def cursor_down
    if row < row_max - 1
      select(@index + col_max)
    else
      select(0)
    end
  end
  # カーソルを上に移動
  def cursor_up
    if row + 1 > 1
      select(@index - col_max)
    else
      select(5)
    end
  end
  # リフレッシュ
  def refresh
    if !@spriteset.nil?
      dispose_image
      set_image_plugin
      set_image_selecter
    end
    super
  end
  # 解放
  def dispose
    super
    dispose_image
    @spriteset.each{|k, spriteset| spriteset.dispose}
    @sprite_selecter.each{|k, sprite_selecter| sprite_selecter.dispose}
    @image_button.dispose
    @spriteset = nil
    @sprite_selecter = nil
    @image_button = nil
    @flasher = nil
  end
  # イメージの解放
  def dispose_image
    begin
      @spriteset.each{|k, spriteset| spriteset.image.dispose}
      @sprite_selecter.each{|k, sprite_selecter| sprite_selecter.image.dispose}
    rescue
    end
  end
  # Zの設定
  def z=(z)
    super(z)
    @spriteset.each{|k, set| set.z = z}
    @sprite_selecter.each{|k, set| set.z = z + 2}
  end
  # ヘルプテキスト更新
  def update_help
    help = $player.equip_plugin[index].nil? ? nil : Data_Plugin.description($player.equip_plugin[index])
    @help_window.set_item(help.nil? ? '装備なし' : help)
  end
  # 決定/キャンセル処理、画面切り替えを追加
  def process_handling
    super
    return call_handler(:switching) if Key.push?(K_A)
    return call_handler(:narrow) if Key.push?(K_S)
  end
end
