# -----------------------------------------------------
# module Battle
# 戦闘の処理を管理するモジュール
# 2019.06.09 レベルの廃止、常にLv.1に
#-----------------------------------------------------

module BattleManager
  ONOM_RESET = 120
  BASE_RELEASE_COUNT = 150
  RELEASE_RESULT_COUNT = 60
  COMMAND_ARY = ["up","down","left","right"]
  def self.setup(id = nil)
    #$player.init_stats
    @turn = 0 # 経過ターン数
    @enemy_id = id
    @scene = nil # SceneManagerとの連携、Scene_Baseの取得用
    @phase = :init # 進行管理用フェーズ
    @in_perform = false # 演出の開始
    @log_window = nil # ログウィンドウとの連携
    @message_window = nil # メッセージウィンドウとの連携
    $enemy = Enemy.new(id) # 敵オブジェクトの生成
    @position_index = 0 # 体位インデックス(データ採集...通常;0 拘束;1,武力行使...通常;0 性技;1 拘束;2)
    @break_wear = false # 衣服の状態(着衣;false 脱衣;true)
    @break_pant = false # パンツの状態(着衣;false 脱衣;true)
    @spriteset = nil # 戦闘画面へのアクセサ
    @force = Data_Enemy.system(id)
    # バトラーオブジェクトの生成
    @battler = {}
    @battler[:player] = Battler.new($player)
    @battler[:enemy]  = Battler.new($enemy)
    @continuity = {}
    @action_order = []
    @target_index = 0
    @bool = true
    @onom_counter = 0
    @release_counter = 0
    @release_command = []
    @release_num = 0
    @release_miss_count = 0
    # @current_exp = nil # 現在の経験値を保持
    @on_simulation = false
    @break_wear_chgpos = true # 装甲破壊時の体位変更許可
    @flag_halfsp = false
    @flag_break_arm = false
    @insulted = false
    @ejection = false
  end
  # チュートリアルの設定
  def self.set_tutorial
    # @current_exp = $player.exp
    # $player.set_exp(0)
    # $player.level = 1
    # $player.init_stats
    @event_lapper = Battle_Events.new(@enemy_id)
  end
  # チュートリアル判定
  def self.on_tutorial
    begin
      return @event_lapper.instance_of?(Battle_Events)
    rescue
      return false
    end
  end
  # チュートリアルの終了
  def self.off_tutorial
    @event_lapper = nil
  end
  # シミュレーション判定
  def self.set_simulation
    # @current_level = level
    @on_simulation = true
  end
  # シミュレーション中か？
  def self.on_simulation?
    return @on_simulation
  end
  # 武力行使判定
  def self.force?
    @force == :force
  end
  # モードの直接指定（回想用）
  def self.set_mode(sym)
    @force = sym
  end
  # モードへのアクセス（回想用）
  def self.get_mode
    return @force
  end
  # 陵辱モード設定
  def self.set_insult=(bool)
    @insulted = bool
  end
  # 陵辱モード中か？
  def self.insulted?
    return @insulted
  end
  # 賢者モード設定
  def self.set_ejection=(bool)
    @ejection = bool
  end
  # 賢者モード中か？
  def self.ejection?
    return @ejection
  end
  # スプライトセットの設定
  def self.spriteset=(spriteset)
    @spriteset = spriteset
  end
  # スプライトセットへのアクセス
  def self.spriteset
    @spriteset
  end
  # スプライトセットの設定
  def self.spriteset_releasecommand=(spriteset)
    @spriteset_releasecommand = spriteset
  end
  # 現在の体位インデックスを取得
  def self.position_index
    return @position_index
  end
  # 現在の体位データを取得
  def self.position
    return EnemyPos.enemy_pos(@enemy_id, @position_index)
  end
  # 現在の体位インデックスを変更
  def self.change_position(index = nil)
    # 立ち絵パターンの拡張用処理
    # @position_index = [[0, index].max, EnemyPos.max_position_size(@enemy_id)]
    # 状態から取得する2パターンのプログラミング
    case @position_index
    when 0
      @position_index = 1
    when 1
      @position_index = 0
    else
      @position_index = 0
    end
    # 中編以降における相手からの拘束を含めた体位の直接選択
    @position_index = index unless index.nil?
    # なんかどっかから拘束状態なのに変更かけてくるので陵辱モード中は強制でpositionを2に
    @position_index = 2 if @insulted
    @spriteset.enemy.position = @position_index
    @battler[:enemy].charge_cancel
    @action_order = [@action_order[@target_index]]
    @target_index = 0
    @phase = :judge
  end
  # 現在の衣服の状態を取得
  def self.break_wear?
    return @break_wear
  end
  # 衣服の状態を変更
  def self.break_wear
    @break_wear ? return : @break_wear = true
    # 本作における2パターンのメタプログラミング
    @spriteset.enemy.wear = 1
  end
  # 現在のパンツの状態を取得
  def self.break_pant?
    return @break_pant
  end
  # パンツの状態を変更
  def self.break_pant
    @break_pant ? return : @break_pant = true
    # 本作における2パターンのメタプログラミング
    @spriteset.enemy.pant = 1
  end
  # 更新
  def self.update
    @event_lapper.update if @event_lapper.instance_of?(Battle_Events)
    self.observe_phase
    if $system.animation
      if ONOM_RESET < @onom_counter
        @spriteset.show_onom
        @onom_counter = 0
      else
        @onom_counter += 1
      end
    end
    @spriteset_releasecommand.update unless @spriteset_releasecommand.nil?
  end
  # フェイズの監視
  def self.observe_phase
    case @phase
    when :init
      self.input_start if !@log_window.busy? && !@message_window.busy?
    when :result
      if !@log_window.busy? && !@message_window.busy?
        unless @action_order[@target_index][1].continuity_count_down < 0
          self.action_result
        else
          @action_order[@target_index][1].delete_result
          self.judge_win_lose
        end
      end
    when :judge
      #return if @log_window.busy? && !@message_window.busy?
      if !@log_window.busy? && !@message_window.busy?
        if @battler[:player].subject.hp <= 0
          self.process_defeat
          return
        elsif @battler[:enemy].subject.hp <= 0
          self.process_victory
          return
        elsif @battler[:player].subject.ep <= 0 && !@ejection
          self.process_ejection
          return
        else
          self.action_end
        end
      end
    when :action_end
      return if self.on_tutorial && Data_Tutorial.get_switch(0) && @enemy_id == 1
      if !@log_window.busy? && !@message_window.busy?
        if @action_order[@target_index].nil?
          self.turn_end
        else
          self.action_start
        end
      end
    when :turn_end
      if self.force? && $enemy.sp <= 0 && @break_wear_chgpos
        self.change_position(1)
        @break_wear_chgpos = false
      end
      self.input_start if !@log_window.busy? && !@message_window.busy?
    when :defeat, :victory
      self.battle_end if !@log_window.busy? && !@message_window.busy?
    when :release
      self.release_rest
    when :release_result
      if @success_image_count <= 0
        if @spriteset_releasecommand.success?
          # リリースコマンド入力成功
          @spriteset_releasecommand.co_clear
          @release_num = 0
          @release_counter = 0
          @release_command = []
          @release_miss_count = 0
          @insulted = false
          self.change_position(1)
          # @spriteset.enemy.expression = @break_wear ? 2 : 0
          self.log_window.display_release_bind(true)
          self.message_window.add_message(
            Data_EnemyMessage.release_success($enemy.id),
            Data_EnemyMessage.image($enemy.id),
            Data_EnemyMessage.side($enemy.id)
          )
          self.input_start
        else
          if self.on_tutorial
            Audio.play_success_release
            @spriteset_releasecommand.success(true)
            @success_image_count = RELEASE_RESULT_COUNT
            @spriteset_releasecommand.success_image_x = -1704
            return
          end
          # リリースコマンド入力失敗
          @spriteset_releasecommand.co_clear
          @release_num = 0
          @release_command = []
          @release_miss_count += 1
          self.log_window.display_release_bind(false)
          self.message_window.add_message(
            Data_EnemyMessage.release_miss($enemy.id),
            Data_EnemyMessage.image($enemy.id),
            Data_EnemyMessage.side($enemy.id)
          )
          @battler[:player].make_action_restrict
          self.turn_start
        end
      else
        @success_image_count -= 1
        if 55 > @success_image_count && @success_image_count > 5
          @spriteset_releasecommand.success_image_x = 1
        else
          @spriteset_releasecommand.success_image_x = 752.5 / 5
        end
      end
    end
  end
  # 拘束解除処理（武力行使）
  def self.release_rest
    if @release_counter <= 0
      Audio.play_miss_release
      @spriteset_releasecommand.success(false)
      @success_image_count = RELEASE_RESULT_COUNT
      @phase = :release_result
    else
      @release_counter -= 1
      if Key.push?(@release_command[@release_num].to_sym)
        if @release_num >= @release_command.size - 1
          Data_Tutorial.set_switch(3, true) if self.on_tutorial
          Audio.play_success_release
          @spriteset_releasecommand.success(true)
          @success_image_count = RELEASE_RESULT_COUNT
          @phase = :release_result
        else
          Audio.play_release
          @release_num += 1
          @spriteset_releasecommand.input_command_update(@release_num)
        end
      else
        COMMAND_ARY.each{|co| @release_counter = 0 if Key.push?(co.to_sym)}
      end
    end
  end
  # 拘束解除コマンド入力時間の残り時間を取得
  def self.release_counter
    return @release_counter
  end
  # バトラーの取得(key = :player/:enemy)
  def self.battler
    return @battler
  end
  # HLA値の一致度を取得
  def self.hla_resemblance
    dec_p = @battler[:player].subject.hla_gradec
    dec_e = @battler[:enemy].subject.hla_gradec
    return ((dec_p - dec_e) / Math::PI).abs
  end
  # シーンの変更
  def self.scene=(scene)
    @scene = scene
  end
  # 敵IDの取得
  def self.enemy_id
    return @enemy_id
  end
  # ターン数の取得
  def self.turn
    return @turn
  end
  # フェイズの取得
  def self.phase
    return @phase
  end
  # ログウィンドウの設定
  def self.log_window=(window)
    @log_window = window
  end
  # ログウィンドウへのアクセス
  def self.log_window
    return @log_window
  end
  # メッセージウィンドウの設定
  def self.message_window=(window)
    @message_window = window
  end
  # メッセージウィンドウへのアクセス
  def self.message_window
    return @message_window
  end
  # 入力の開始
  def self.input_start
    @spriteset.enemy.attack_expression = 0 unless @insulted
    @phase = :input
    if @battler[:player].restrict?
      @battler[:player].make_action_restrict
      self.turn_start
    else
      if @position_index == 2
        self.start_release_command
      else
        @scene.input_start
      end
    end
  end
  # ターンの開始処理
  def self.turn_start
    # 行動順の決定
    @turn += 1
    @spriteset.enemy.attack_expression = 0 unless @insulted
    @target_index = 0
    @phase = :turn_start
    if @battler[:enemy].restrict? || (self.on_tutorial && @turn == 3 && @enemy_id == 79)
      @battler[:enemy].make_action_restrict
    else
      @battler[:enemy].make_enemy_action
    end
    @action_order = []
    @action_order = @battler.sort{|(k1, v1), (k2, v2)|
      v2.speed <=> v1.speed
    }
    self.action_start
  end
  # 拘束状態の解除コマンド入力開始（武力行使）
  def self.start_release_command
    @phase = :release
    6.times{@release_command << COMMAND_ARY[rand(4)]}
    @spriteset_releasecommand.create_command(@release_command)
    self.make_count_time
  end
  # 解除コマンド入力の制限時間を計算
  def self.make_count_time
    #難易度を考慮したベースリリースカウントの算出
    dif_base_re_cnt = 30 * $system.game_level + BASE_RELEASE_COUNT
    begin
      #ステータス補正値計算
      @release_counter = (($player.agi - $enemy.agi)+($player.luk - $enemy.luk)+($player.dfn - $enemy.dfn)).to_f / 100
      @release_counter = dif_base_re_cnt + dif_base_re_cnt * @release_counter
      @release_counter = @release_counter.to_i - (@release_miss_count * 10)
    rescue
      return dif_base_re_cnt
    end
  end
  # 行動の開始処理
  def self.action_start
    @phase = :action
    unless @action_order[@target_index][1].subject.instance_of?(Game_Player) && @insulted
      unless @action_order[@target_index][1].restrict?
        unless @action_order[@target_index][1].subject.instance_of?(Enemy) && self.on_tutorial && @turn == 3 && @enemy_id == 79
          @action_order[@target_index][1].act
        end
      end
    end
    @battler.each{|k, v| v.refresh_stats}
    @phase = :result
  end
  # 行動結果の処理
  def self.action_result
    @action_order[@target_index][1].apply_result unless @action_order[@target_index][1].restrict?
    @battler.each{|k, v| v.refresh_stats}
  end
  # 行動結果のクリア
  def self.action_clear_for_chgpos
    @phase = :judge
    @action_order = []
  end
  # 勝敗判定処理
  def self.judge_win_lose
    @phase = :judge
  end
  # 行動の終了
  def self.action_end
    # --- 武力行使モードの処理 --- #
    if self.force?
      # 装甲値が50%以下になった場合、メッセージ表示
      unless @flag_halfsp || @battler[:enemy].subject.sp >= @battler[:enemy].subject.msp / 2
        self.message_window.add_message(
          Data_EnemyMessage.halfsp($enemy.id),
          Data_EnemyMessage.image($enemy.id),
          Data_EnemyMessage.side($enemy.id)
        )
        @flag_halfsp = true
      end
      # 装甲破壊した場合、性技戦闘モーションに変更
      if @position_index == 0 && self.break_wear? && !@flag_break_arm
        self.change_position(1)
        self.log_window.display_pos_change($enemy)
        self.message_window.add_message(
          Data_EnemyMessage.chgpos($enemy.id, :action),
          Data_EnemyMessage.image($enemy.id),
          Data_EnemyMessage.side($enemy.id)
        )
        @flag_break_arm = true
      end
    end
    # -------------------------- #
    @spriteset.enemy.attack_expression = 0 unless @insulted
    @action_order[@target_index][1].process_action_end unless @action_order[@target_index].nil?
    @target_index += 1
    @phase = :action_end
  end
  # ターンの終了処理
  def self.turn_end
    @phase = :turn_end
    # 陵辱モードでない場合と陵辱モードで射精モーションである場合は待機モーションに戻す
    unless @insulted
      @spriteset.clear_balloon
      @spriteset.enemy.attack_expression = 0
    else
      if @spriteset.enemy.attack == 2 || @spriteset.enemy.attack == 3
        @spriteset.clear_balloon
        @spriteset.enemy.attack_expression = 0
      end
    end
    @turn += 1
    @battler.each{|k, battler| battler.forward_turn}
    @battler.each{|k, battler| battler.clear_action}
  end
  # 射精の処理
  def self.process_ejection
    self.log_window.clear
    @battler[:player].ejection(@battler[:enemy].skill_id_accecer)
  end
  # 全滅の処理
  def self.process_defeat
    @phase = :defeat
    self.log_window.clear
    self.log_window.display_defeat
    self.message_window.add_message(
      Data_EnemyMessage.defeat($enemy.id),
      Data_EnemyMessage.image($enemy.id),
      Data_EnemyMessage.side($enemy.id)
    )
  end
  # 勝利の処理
  def self.process_victory
    @phase = :victory
    self.message_window.clear_message
    self.log_window.clear
    @log_window.display_victory($enemy)
    unless @on_simulation
      get_dropitem(Data_Enemy.drop(@enemy_id))
      $player.victory_count += 1
      case $player.victory_count
      when 1  ; $player.trophy << 24
      when 5  ; $player.trophy << 25
      when 10 ; $player.trophy << 26
      when 20 ; $player.trophy << 27
      when 30 ; $player.trophy << 28
      when 40 ; $player.trophy << 29
      when 50 ; $player.trophy << 30
      end
      # 中編勝利回数称号
      $player.trophy << 69 if $player.victory_count >= 60 && !$player.trophy.include?(69)
      $player.trophy << 70 if $player.victory_count >= 70 && !$player.trophy.include?(70)
      $player.trophy << 71 if $player.victory_count >= 80 && !$player.trophy.include?(71)
      # 後編勝利回数称号
      $player.trophy << 109 if $player.victory_count >= 90 && !$player.trophy.include?(109)
      $player.trophy << 110 if $player.victory_count >= 100 && !$player.trophy.include?(110)
      # ヘルメス駐屯地イベ称号用　#
      $game_switches[1955] = true if $enemy.id == 134 #兵士
      $game_switches[1956] = true if $enemy.id == 135 #看護師
      $player.trophy << 115 if $game_switches[1955] && $game_switches[1956]
    end
  end
  # 戦闘の終了処理
  def self.battle_end
    if self.force?
      @spriteset_releasecommand.dispose
      @spriteset_releasecommand = nil
    end
    case @phase
    when :victory
      Audio.stop_bgm
      Audio.delete_bgm(:enemy)
      Audio.play_bgm_list(:map) if BattleManager.on_simulation?
      @phase = nil
      $game_switches[10000 + $enemy.id.to_i] = false
      self.log_window.clear
      @on_simulation = false
      ScreenShot.dispose
      SceneManager.return
    when :defeat
      $game_switches[10000 + $enemy.id.to_i] = true unless BattleManager.on_simulation?
      if $system.lose_scene == 2
        @phase = nil
        self.log_window.clear
        ScreenShot.dispose
        SceneManager.return
        Audio.stop_bgm
        Audio.delete_bgm(:enemy)
        Audio.play_bgm_list(:map) if BattleManager.on_simulation?
      else
        @phase = :recollect
        self.log_window.clear
        SceneManager.goto(Scene_Recollect, :black, @spriteset.enemy, @spriteset.onom)
      end
      unless @on_simulation
        $player.defeat_count += 1
        $game_variables[11] = $player.defeat_count
        case $player.defeat_count
        when 1  ; $player.trophy << 31
        when 5  ; $player.trophy << 32
        when 10 ; $player.trophy << 33
        when 20 ; $player.trophy << 34
        when 30 ; $player.trophy << 35
        when 40 ; $player.trophy << 36
        when 50 ; $player.trophy << 37
        end
        # 中編敗北回数称号
        $player.trophy << 72 if $player.defeat_count >= 60 && !$player.trophy.include?(72)
        $player.trophy << 73 if $player.defeat_count >= 70 && !$player.trophy.include?(73)
        $player.trophy << 74 if $player.defeat_count >= 80 && !$player.trophy.include?(74)
        # 後編勝利回数称号
        $player.trophy << 111 if $player.defeat_count >= 90 && !$player.trophy.include?(111)
        $player.trophy << 112 if $player.defeat_count >= 100 && !$player.trophy.include?(112)
      end
    end
    $player.init_stats
  end
  # 戦闘中か？
  def self.in_battle?
    return !@phase.nil?
  end
  # 現在のスプライトを取得
  def self.get_spriteset
    @spriteset
  end
  # ドロップアイテムの取得
  def self.get_dropitem(items)
    items.each{|item| get_item(item[0], item[1]) if rand(100) <= item[2]}
  end
  # アイテム取得処理
  def self.get_item(category, item_id)
    case category
    when :i
      $player.give_item(item_id)
      @log_window.display_obtainitem(Data_Item.name(item_id))
    when :p
      $player.give_plugin(item_id)
      @log_window.display_obtainitem(Data_Plugin.name(item_id))
    end
  end
end
