#-----------------------------------------------------
# module Fonts
# ゲーム内で使用されるフォントオブジェクトを管理するモジュール
#-----------------------------------------------------

module Fonts
  # 初期化@DataManager#start
  def self.init
    Font.install($game_directory + '/font/font_1_kokugl_1.15_rls.ttf')
    @fonts = {}
    @fonts[:default] = []     # 通常
    @fonts[:bold] = []        # 太字
    @fonts[:italic] = []      # 斜体
    @fonts[:italic_bold] = [] # 太字斜体
    (minsize_font..maxsize_font).each{|size|
      @fonts[:default][size]     = Font.new(size, default_font_name, {auto_fitting: auto_fitting})
      @fonts[:bold][size]        = Font.new(size, default_font_name, {weight: true, auto_fitting: auto_fitting})
      @fonts[:italic][size]      = Font.new(size, default_font_name, {italic: true, auto_fitting: auto_fitting})
      @fonts[:italic_bold][size] = Font.new(size, default_font_name, {italic: true, weight: true, auto_fitting: auto_fitting})
    }
  end
  # フォントデータの取得
  def self.get(size = Const::DefaultFontSize, option = :default)
    begin
      font = @fonts[option][size]
    rescue
      puts "[INFO] Invalid Fonts is specified."
      puts "[INFO] Dafault Font was returned."
      return default_font
    end
    if font.nil?
      puts "[INFO] There is no font of the specified size."
      puts "[INFO] Dafault Font was returned."
      return default_font
    else
      return font
    end
  end
  # 指定されたサイズが描画されるフォントサイズを意味するか
  def self.auto_fitting
    return true
  end
  # 標準フォントオブジェクトの取得
  def self.default_font
    @fonts[:default][Const::DefaultFontSize]
  end
  # デフォルトフォントネーム
  def self.default_font_name
    f = open($game_directory + '/font/name.txt')
    item = f.read
    f.close
    return item
    #return "刻ゴシック Light"
  end
  # 最小フォントサイズ
  def self.minsize_font
    return 5
  end
  # 最大フォントサイズ
  def self.maxsize_font
    return 75
  end
end
